@@ -1,6 +1,6 @@
 # -*- python -*-
 #
-# Copyright (C) 2001,2002 Jason R. Mastaler <jason@mastaler.com>
+# Copyright (C) 2001,2002,2003 Jason R. Mastaler <jason@mastaler.com>
 #
 # This file is part of TMDA.
 #
@@ -147,6 +147,8 @@ if not vars().has_key('MAIL_TRANSFER_AGENT'):
 # DELIVERY = "~/Maildir/"
 # DELIVERY = "~/Mailbox"
 # DELIVERY = "/var/mail/jasonrm"
+# DELIVERY = ":~/Mailbox"
+# DELIVERY = ":/var/spool/mail/jasonrm"
 # DELIVERY = "|/usr/bin/maildrop"
 # DELIVERY = "|/usr/bin/procmail ~/.procmailrc-tmda"
 # DELIVERY = "me@new.job.com"
@@ -163,7 +165,7 @@ if not vars().has_key('DELIVERY'):
 # A single character which specifies the separator between user names
 # and address extensions (e.g, user-ext).
 # The default under qmail is `-', while the default for Sendmail and
-# friends is likely `+'.
+# friends is likely `+'. The default for MMDF is '='.
 #
 # Default is "-"
 if not vars().has_key('RECIPIENT_DELIMITER'):
@@ -176,7 +178,7 @@ if not vars().has_key('RECIPIENT_DELIMITER'):
 if not vars().has_key('ALLOW_MODE_640'):
     ALLOW_MODE_640 = 0
 
-# OUTGOINGMAIL
+# MAIL_TRANSPORT
 # Final delivery method for all outgoing mail (server and client).
 # Possible values include:
 # 
@@ -187,22 +189,14 @@ if not vars().has_key('ALLOW_MODE_640'):
 #
 # "sendmail"
 #    Deliver messages via the command line interface to the sendmail
-#    program (/usr/sbin/sendmail).  Use at your own risk.  "smtp" is
-#    highly recommended.
+#    program (e.g, /usr/sbin/sendmail).
 #
-#    SECURITY WARNING: The 'sendmail' method is not secure.  Because
-#    this method uses popen(), it goes through the shell.  It does not
-#    scan the arguments for potential exploits and so it should be
-#    considered unsafe.  For performance reasons, it's not recommended
-#    either -- use the 'smtp' method instead, even if
-#    MAIL_TRANSFER_AGENT is "sendmail".
-#
-# Default is "smtp"
-if not vars().has_key('OUTGOINGMAIL'):
-    OUTGOINGMAIL = "smtp"
+# Default is "sendmail"
+if not vars().has_key('MAIL_TRANSPORT'):
+    MAIL_TRANSPORT = "sendmail"
 
 # SMTPHOST
-# SMTP host and optional port, when OUTGOINGMAIL is "smtp".
+# SMTP host and optional port, when MAIL_TRANSPORT is "smtp".
 # If the hostname or IP address ends with a colon (":") followed by a
 # number, that suffix will be stripped off and the number interpreted
 # as the port number to use.  Otherwise, the standard SMTP port (25)
@@ -216,7 +210,7 @@ if not vars().has_key('OUTGOINGMAIL'):
 # SMTPHOST = "mailhost.company.com:1234"
 #
 # Default is "localhost" (port 25 on the local host)
-if not vars().has_key('SMTPHOST') and OUTGOINGMAIL == 'smtp':
+if not vars().has_key('SMTPHOST') and MAIL_TRANSPORT == 'smtp':
     SMTPHOST = "localhost"
 
 # SMTPAUTH_USERNAME
@@ -229,7 +223,7 @@ if not vars().has_key('SMTPHOST') and OUTGOINGMAIL == 'smtp':
 # SMTPAUTH_USERNAME = "johndoe"
 #
 # No default.
-if not vars().has_key('SMTPAUTH_USERNAME') and OUTGOINGMAIL == 'smtp':
+if not vars().has_key('SMTPAUTH_USERNAME') and MAIL_TRANSPORT == 'smtp':
     SMTPAUTH_USERNAME = None
 
 # SMTPAUTH_PASSWORD
@@ -242,7 +236,7 @@ if not vars().has_key('SMTPAUTH_USERNAME') and OUTGOINGMAIL == 'smtp':
 # SMTPAUTH_PASSWORD = "6Yu_9iKzs"
 #
 # No default.
-if not vars().has_key('SMTPAUTH_PASSWORD') and OUTGOINGMAIL == 'smtp':
+if not vars().has_key('SMTPAUTH_PASSWORD') and MAIL_TRANSPORT == 'smtp':
     SMTPAUTH_PASSWORD = None
 
 # SMTPSSL
@@ -252,7 +246,7 @@ if not vars().has_key('SMTPAUTH_PASSWORD') and OUTGOINGMAIL == 'smtp':
 # support.  Requires Python 2.2 or greater.
 #
 # Default is 0 (turned off)
-if not vars().has_key('SMTPSSL') and OUTGOINGMAIL == 'smtp':
+if not vars().has_key('SMTPSSL') and MAIL_TRANSPORT == 'smtp':
     SMTPSSL = 0
 
 # SMTPSSL_KEYFILE
@@ -261,7 +255,7 @@ if not vars().has_key('SMTPSSL') and OUTGOINGMAIL == 'smtp':
 #
 # No default.
 if not vars().has_key('SMTPSSL_KEYFILE') and \
-       OUTGOINGMAIL == 'smtp' and SMTPSSL:
+       MAIL_TRANSPORT == 'smtp' and SMTPSSL:
     SMTPSSL_KEYFILE = None
 
 # SMTPSSL_CERTFILE
@@ -271,12 +265,12 @@ if not vars().has_key('SMTPSSL_KEYFILE') and \
 #
 # No default.
 if not vars().has_key('SMTPSSL_CERTFILE') and \
-       OUTGOINGMAIL == 'smtp' and SMTPSSL:
+       MAIL_TRANSPORT == 'smtp' and SMTPSSL:
     SMTPSSL_CERTFILE = None
 
 # SMTP_MAX_SESSIONS_PER_CONNECTION
 # An integer specifying a ceiling on the number of SMTP sessions to
-# perform on a single socket connection, when OUTGOINGMAIL is
+# perform on a single socket connection, when MAIL_TRANSPORT is
 # "smtp".  Some MTAs have limits.  Set this to 0 to do as many
 # as we like (i.e. your MTA has no limits).  Set this to some number
 # great than 0 and TMDA will close the SMTP connection and re-open it
@@ -284,14 +278,14 @@ if not vars().has_key('SMTPSSL_CERTFILE') and \
 #
 # Default is 0
 if not vars().has_key('SMTP_MAX_SESSIONS_PER_CONNECTION') and \
-       OUTGOINGMAIL == 'smtp':
+       MAIL_TRANSPORT == 'smtp':
     SMTP_MAX_SESSIONS_PER_CONNECTION = 0
 
 # SENDMAIL_PROGRAM
 # The path to the sendmail program, or sendmail compatibility
-# interface when OUTGOINGMAIL is "sendmail".
+# interface when MAIL_TRANSPORT is "sendmail".
 # Defaults to one of the two standard locations.
-if not vars().has_key('SENDMAIL_PROGRAM') and OUTGOINGMAIL == 'sendmail':
+if not vars().has_key('SENDMAIL_PROGRAM') and MAIL_TRANSPORT == 'sendmail':
     for sendmail in ('/usr/sbin/sendmail', '/usr/lib/sendmail'):
         if os.path.exists(sendmail):
             SENDMAIL_PROGRAM = sendmail
@@ -323,7 +317,7 @@ if not vars().has_key('VIRTUALDOMAINS'):
 #
 # Default is an empty envelope sender <>.
 if not vars().has_key('BOUNCE_ENV_SENDER'):
-    if OUTGOINGMAIL == 'sendmail' and \
+    if MAIL_TRANSPORT == 'sendmail' and \
            MAIL_TRANSFER_AGENT in ('qmail', 'postfix'):
         # qmail/Postfix's /usr/sbin/sendmail doesn't like -f '<>'
         BOUNCE_ENV_SENDER = ''
@@ -430,6 +424,14 @@ if not vars().has_key('CGI_SETTINGS'):
 if not vars().has_key('CGI_URL'):
     CGI_URL = None
 
+# CGI_VIRTUALUSER
+# Set this variable to 1 if you wish to enable tmda-cgi's "virtual
+# user" support (http://tmda.net/tmda-cgi/virtual.html).
+#
+# Default is 0 (disabled)
+if not vars().has_key('CGI_VIRTUALUSER'):
+    CGI_VIRTUALUSER = 0
+
 # CONFIRM_ADDRESS
 # An optional e-mail address to use for creating confirmation
 # addresses.  Normally, the recipient address is used, but in some
@@ -542,6 +544,40 @@ if not vars().has_key('CONFIRM_MAX_MESSAGE_SIZE'):
 if not vars().has_key('TEMPLATE_DIR'):
     TEMPLATE_DIR = None
 
+# TEMPLATE_DIR_MATCH_RECIPIENT
+# Set this variable to 1 if you want to use specific templates for
+# different recipient addresses.  Be sure to set TEMPLATE_DIR as well.
+#
+# The TEMPLATE_DIR_MATCH_RECIPIENT feature enables TMDA to search for
+# customized template files based on the recipient address.  To continue
+# with the example given for TEMPLATE_DIR_MATCH_SENDER, if a message
+# arrives for foo-lists-guitar@bar.baz.de then TMDA will search for
+# template files in the following directories:
+#
+#  foo-lists-guitars@bar.baz.de/
+#
+#  foo-lists@bar.baz.de/
+#
+#  foo@bar.baz.de/
+#
+#  bar.baz.de/
+#
+#  baz.de/
+#
+#  de/
+#
+# This example assumes that RECIPIENT_DELIMITER is set to `-'.  This
+# feature also works for *-confirm-* addresses.
+#
+# When both TEMPLATE_DIR_MATCH_RECIPIENT and TEMPLATE_DIR_MATCH_SENDER
+# are enabled, the TEMPLATE_DIR_MATCH_RECIPIENT directories are searched
+# after the sender directories.
+#
+# Default is 0 (turned off)
+
+if not vars().has_key('TEMPLATE_DIR_MATCH_RECIPIENT'):
+    TEMPLATE_DIR_MATCH_RECIPIENT = 0
+
 # TEMPLATE_DIR_MATCH_SENDER
 # Set this variable to 1 if you want to use sender specific template
 # directory matching. Make sure you also have TEMPLATE_DIR set.
@@ -990,61 +1026,128 @@ if not vars().has_key('RESPONSE_DIR') and MAX_AUTORESPONSES_PER_DAY != 0:
 if not vars().has_key('AUTORESPONSE_INCLUDE_SENDER_COPY'):
     AUTORESPONSE_INCLUDE_SENDER_COPY = 2
 
-# MYSQL_ENABLED
-# All defaults beginning with "MYSQL_" configure the use of a MySQL
-# database in the filtering process.  To use a MySQL database, you
-# must have the _mysql library loaded.  See:
-# http://sourceforge.net/projects/mysql-python/
-#
-# MYSQL_ENABLED must be set to 1 if you wish to use a MySQL database
-# in your filters.
+# DB_CONNECTION
+# Python DB API Connection object. This is specific to the database
+# and should be created in the ~/.tmda/config file.  This is typically
+# created by importing the appropriate module and calling the connect()
+# function.
 #
 # Example:
-# MYSQL_ENABLED = 1
+# import MySQLdb
+# DB_CONNECTION = MySQLdb.connect("...")
 #
-# Default is 0 (disabled)
-if not vars().has_key('MYSQL_ENABLED'):
-    MYSQL_ENABLED = 0
+# Default is None
+if not vars().has_key('DB_CONNECTION'):
+    DB_CONNECTION = None
 
-# MYSQL_HOST
-# MySQL host to connect to.
+# DB_CONFIRM_APPEND
+# SQL INSERT statement to be used to insert confirmed sender addresses
+# into a SQL database. The Python DB API will take care of properly
+# quoting parameters that are strings.
+# Requires a valid DB_CONNECTION object.
 #
-# Example:
-# MYSQL_HOST = "db.my.server"
+# Available substition parameters are:
 #
-# Default is "localhost".
-if MYSQL_ENABLED and not vars().has_key('MYSQL_HOST'):
-    MYSQL_HOST = "localhost"
+# %(recipient)s  - USERNAME@HOSTNAME
+# %(username)s   - USERNAME
+# %(hostname)s   - HOSTNAME
+# %(sender)s     - sender's address (envelope sender or X-Primary-Address)
+#
+# Examples:
+#
+# DB_CONFIRM_APPEND = """
+#  INSERT INTO whitelist (user_email, address)
+#       VALUES (%(recipient)s, %(sender)s)"""
+#
+# DB_CONFIRM_APPEND = """
+#  INSERT INTO wildcard_list (uid, address, action)
+#       SELECT uid, %(sender)s, 'accept'
+#         FROM users
+#        WHERE users.email = %(recipient)s"""
+#
+# Default is None
+if not vars().has_key('DB_CONFIRM_APPEND'):
+    DB_CONFIRM_APPEND = None
 
-# MYSQL_DATABASE
-# MySQL database containing any lists used in filter.
+# DB_BARE_APPEND
+# SQL INSERT statement to be used to insert recipient addresses into
+# a SQL database if the outgoing <action> was 'bare=append'. The Python
+# DB API will take care of properly quoting parameters that are strings.
+# Requires a valid DB_CONNECTION object.
 #
-# Example:
-# MYSQL_DATABASE = "JimsTMDA"
+# Available substition parameters are:
 #
-# Default is "TMDA".
-if MYSQL_ENABLED and not vars().has_key('MYSQL_DATABASE'):
-    MYSQL_DATABASE = "TMDA"
+# %(recipient)s  - recipient's email address
+# %(username)s   - USERNAME (of TMDA user)
+# %(hostname)s   - HOSTNAME (of TMDA user)
+# %(sender)s     - USERNAME@HOSTNAME (address of TMDA user)
+# %(fromheader)s - address of TMDA user in From: header field
+#
+# Examples:
+#
+# DB_BARE_APPEND = """
+#  INSERT INTO whitelist (user_email, address)
+#       VALUES (%(sender)s, %(recipient)s)"""
+#
+# Default is None
+if not vars().has_key('DB_BARE_APPEND'):
+    DB_BARE_APPEND = None
 
-# MYSQL_USER
-# MySQL user to use when connecting to database.
+# PENDING_DIR
+# Full path to the directory containing messages pending confirmation
+# (aka, the "pending queue").  If this directory doesn't exist, it
+# will automatically be created by TMDA with 0700 permissions when the
+# first message arrives.
 #
 # Example:
-# MYSQL_USER = "Jim"
+# PENDING_DIR = "/full/path/to/pending/"
 #
-# Default is "TMDA".
-if MYSQL_ENABLED and not vars().has_key('MYSQL_USER'):
-    MYSQL_USER = "TMDA"
+# Default is ~/.tmda/pending/
+if not vars().has_key('PENDING_DIR'):
+    PENDING_DIR = os.path.join(DATADIR, 'pending')
 
-# MYSQL_PASSWORD
-# MySQL password to use when connecting to database.
+# PENDING_LIFETIME
+# A time interval describing how long a message can live in the
+# pending queue before it's subject to automated deletion by
+# tmda-filter (a feature controlled by the PENDING_CLEANUP_ODDS
+# setting).
 #
-# Example:
-# MYSQL_USER = "G@rfi31d"
+# Time intervals can be expressed in seconds (s), minutes (m), hours
+# (h), days (d), weeks (w), months (M), or years (Y).
 #
-# Default is "".
-if MYSQL_ENABLED and not vars().has_key('MYSQL_PASSWORD'):
-    MYSQL_PASSWORD = ""
+# Examples:
+#
+# PENDING_LIFETIME = "24h" # messages can live for 24 hours
+# PENDING_LIFETIME = "1M"  # messages can live for 1 month
+#
+# Default is 14d (14 day lifetime)
+if not vars().has_key('PENDING_LIFETIME'):
+    PENDING_LIFETIME = '14d'
+
+# PENDING_CLEANUP_ODDS
+# A floating point number which describes the odds that tmda-filter
+# will automatically clean the pending queue of expired messages upon
+# receipt of an incoming message.  The lifetime of a message in the
+# pending queue is controlled by the PENDING_LIFETIME setting.
+#
+# If you wish to disable this feature in order to clean the queue by
+# hand, or through cron, set this value to 0.0.  If you wish to
+# trigger a cleanup every time a message arrives, set it to 1.0.
+#
+# The closer this value gets to 1.0, the fewer messages you'll have in
+# your pending queue beyond PENDING_LIFETIME, but at some additional
+# expense upon delivery.
+#
+# Examples:
+#
+# PENDING_CLEANUP_ODDS = 0.0   # 0% chance
+# PENDING_CLEANUP_ODDS = 1.0   # 100% chance
+# PENDING_CLEANUP_ODDS = 0.025 # 2.5% chance
+#
+# Default is 0.01, or 1% chance of cleanup for every message received,
+# or cleanup approximately once per 100 messages received.
+if not vars().has_key('PENDING_CLEANUP_ODDS'):
+    PENDING_CLEANUP_ODDS = 0.01
 
 # PENDING_CACHE
 # Path to the cache file used when tmda-pending is invoked with the
@@ -1052,7 +1155,7 @@ if MYSQL_ENABLED and not vars().has_key('MYSQL_PASSWORD'):
 #
 # Default is ~/.tmda/pending/.msgcache
 if not vars().has_key('PENDING_CACHE'):
-    PENDING_CACHE = os.path.join(DATADIR, 'pending', '.msgcache')
+    PENDING_CACHE = os.path.join(PENDING_DIR, '.msgcache')
     
 # PENDING_CACHE_LEN
 # An integer which specifies the maximum number of entries held by
@@ -1077,7 +1180,9 @@ if not vars().has_key('PENDING_BLACKLIST_APPEND'):
 
 # PENDING_DELETE_APPEND
 # Filename to which a sender's e-mail address should be automatically
-# appended when a message is "deleted" by tmda-pending.
+# appended when a message is "deleted" by tmda-pending.  tmda-filter's
+# automated pending queue cleanup feature (see PENDING_CLEANUP_ODDS)
+# also respects this setting.
 #
 # Example:
 # PENDING_DELETE_APPEND = "/full/path/to/blacklist"
@@ -1187,23 +1292,48 @@ if not vars().has_key('ADDED_HEADERS_SERVER'):
 if not vars().has_key('PRIMARY_ADDRESS_MATCH'):
     PRIMARY_ADDRESS_MATCH = 5
 
-# PURGED_HEADERS
+# PURGED_HEADERS_CLIENT
 # A list containing one or more message headers that should be removed
 # from outgoing client-side messages (i.e, messages sent with
 # tmda-sendmail) prior to injection.  Listed headers are
 # case-insensitive, and the purging will only be attempted if the
 # header actually exists.
 #
-# NOTE: PURGED_HEADERS is run _after_ ADDED_HEADERS (see above).
+# NOTE: PURGED_HEADERS_CLIENT is run _after_ ADDED_HEADERS (see above).
 #
 # Examples:
 #
-# PURGED_HEADERS = ["bcc", "resent-bcc", "x-mailer"]
-# PURGED_HEADERS = ["bcc", "resent-bcc", "x-mailer", "user-agent"]
+# PURGED_HEADERS_CLIENT = ["bcc", "resent-bcc", "x-mailer"]
+# PURGED_HEADERS_CLIENT = ["bcc", "resent-bcc", "x-mailer", "user-agent"]
 #
 # Default is "Bcc:" and "Resent-Bcc"
-if not vars().has_key('PURGED_HEADERS'):
-    PURGED_HEADERS = ["bcc", "resent-bcc"]
+if not vars().has_key('PURGED_HEADERS_CLIENT'):
+    PURGED_HEADERS_CLIENT = ["bcc", "resent-bcc"]
+
+# PURGED_HEADERS_SERVER
+# A list containing one or more message headers that should be removed
+# from _all_ outgoing server-side messages (i.e, messages sent with
+# tmda-filter) prior to injection.  Listed headers are case-insensitive
+# and the purging will only be attempted if the header actually exists.
+#
+# See PURGED_HEADERS_CLIENT (above) for some examples.
+#
+# No default
+if not vars().has_key('PURGED_HEADERS_SERVER'):
+    PURGED_HEADERS_SERVER = None
+
+# PURGED_HEADERS_DELIVERY
+# A list containing one or more message headers that should be removed
+# from _all_ delivered messages (i.e, messages stored in an mbox or a
+# maildir, forwarded or passed to a program by TMDA).  Listed headers
+# are case-insensitive and the purging will only be attempted if the
+# header actually exists.
+#
+# See PURGED_HEADERS_CLIENT (above) for some examples.
+#
+# No default
+if not vars().has_key('PURGED_HEADERS_DELIVERY'):
+    PURGED_HEADERS_DELIVERY = None
 
 # RECIPIENT_HEADER
 # A string containing the name of a header (case-insensitive) whose
