@@ -1,6 +1,6 @@
 # -*- python -*-
 #
-# Copyright (C) 2001,2002 Jason R. Mastaler <jason@mastaler.com>
+# Copyright (C) 2001,2002,2003,2004 Jason R. Mastaler <jason@mastaler.com>
 #
 # This file is part of TMDA.
 #
@@ -23,7 +23,6 @@
 
 
 from cStringIO import StringIO
-from email.Generator import Generator
 from string import whitespace as WHITESPACE
 import cPickle
 import email
@@ -36,7 +35,6 @@ import pwd
 import re
 import socket
 import stat
-import string
 import sys
 import tempfile
 import time
@@ -50,6 +48,7 @@ MODE_EXEC = 01
 MODE_READ = 04
 MODE_WRITE = 02
 NL = '\n'
+POSIX_NAME_MAX = 255                    # maximum length of a file name
 
 
 def gethostname():
@@ -63,15 +62,15 @@ def gethostname():
 
 
 def getfullname():
-    """The user's personal name"""
+    """The user's personal name.  Default is an empty value."""
     fullname = os.environ.get('TMDANAME') or \
                os.environ.get('QMAILNAME') or \
                os.environ.get('NAME') or \
                os.environ.get('MAILNAME')
     if not fullname:
         fullname = pwd.getpwuid(os.getuid())[4]
-    if not fullname:
-        fullname = 'Jane Doe'
+        if fullname:
+            fullname = fullname.split(',')[0]
     return fullname
 
 
@@ -129,6 +128,43 @@ def getfileuid(path):
     return statinfo[stat.ST_UID]
 
 
+def getvdomainprepend(address, vdomainsfile):
+    ret_prepend = ''
+    if os.path.exists(vdomainsfile):
+        fp = open(vdomainsfile, 'r')
+        # Parse the virtualdomains control file; see qmail-send(8) for
+        # syntax rules.  All this because qmail doesn't store the original
+        # envelope recipient in the environment.
+        u, d = address.split('@', 1)
+        ousername = u.lower()
+        odomain = d.lower()
+        for line in fp.readlines():
+            vdomain_match = 0
+            line = line.strip().lower()
+            # Comment or blank line?
+            if line == '' or line[0] in '#':
+                continue
+            vdomain, prepend = line.split(':', 1)
+            # domain:prepend
+            if vdomain == odomain:
+                vdomain_match = 1
+            # .domain:prepend (wildcard)
+            elif vdomain[:1] == '.' and odomain.find(vdomain) != -1:
+                vdomain_match = 1
+            # user@domain:prepend
+            else:
+                try:
+                    if vdomain.split('@', 1)[1] == odomain:
+                        vdomain_match = 1
+                except IndexError:
+                    pass
+            if vdomain_match:
+                ret_prepend = prepend
+                break
+        fp.close()
+    return ret_prepend
+
+
 def getvuserhomedir(user, domain, script):
     """Return the home directory of a qmail virtual domain user."""
     cmd = "%s %s %s" % (script, user, domain)
@@ -230,9 +266,9 @@ def unixdate(timesecs=None):
         timesecs = time.time()
     timetuple = time.localtime(timesecs)
     tzname = time.tzname[timetuple[-1]]
-    asctime_list = string.split(time.asctime(timetuple))
+    asctime_list = time.asctime(timetuple).split()
     asctime_list.insert(len(asctime_list)-1, tzname)
-    return string.join(asctime_list)
+    return ' '.join(asctime_list)
 
 
 def make_msgid(timesecs=None, pid=None):
@@ -253,7 +289,8 @@ def make_msgid(timesecs=None, pid=None):
     idhost = os.environ.get('TMDAIDHOST') or \
              os.environ.get('QMAILIDHOST') or \
              gethostname()
-    msgid = '<%s.%s.TMDA@%s>' % (int(timesecs), pid, idhost)
+    idtag = os.environ.get('TMDAIDTAG') or 'TMDA'
+    msgid = '<%s.%s.%s@%s>' % (int(timesecs), pid, idtag, idhost)
     return msgid
 
 
@@ -274,14 +311,14 @@ def make_date(timesecs=None):
 def file_to_dict(file, dict):
     """Process and add then each line of a textfile to a dictionary."""
     for line in fileinput.input(file):
-        line = string.strip(line)
+        line = line.strip()
         # Comment or blank line?
         if line == '' or line[0] in '#':
             continue
         else:
-            fields = string.split(line)
+            fields = line.split()
             key = fields[0]
-            key = string.lower(key)
+            key = key.lower()
             value = fields[1]
             dict[key] = value
     return dict
@@ -291,14 +328,12 @@ def file_to_list(file):
     """Process and then append each line of file to list."""
     list = []
     for line in fileinput.input(file):
-        line = string.strip(line)
+        line = line.strip()
         # Comment or blank line?
         if line == '' or line[0] in '#':
             continue
         else:
-            line = string.expandtabs(line)
-            line = string.split(line, ' #')[0]
-            line = string.strip(line)
+            line = line.expandtabs().split('#')[0].strip()
             list.append(line)
     return list
 
@@ -375,16 +410,18 @@ def append_to_file(str, fullpathname):
     """Append a string to a text file if it isn't already in there."""
     if os.path.exists(fullpathname):
         for line in fileinput.input(fullpathname):
-            line = string.lower(string.strip(line))
+            line = line.strip().lower()
             # Comment or blank line?
             if line == '' or line[0] in '#':
                 continue
             else:
-                if string.lower(string.strip(str)) == line:
+                line = line.expandtabs().split('#')[0].strip()
+                bare = str.expandtabs().split('#')[0].strip()
+                if bare.lower() == line:
                     fileinput.close()
                     return 0
     file = open(fullpathname, 'a+')
-    file.write(string.strip(str) + '\n')
+    file.write(str.strip() + '\n')
     file.close()
 
 
@@ -413,10 +450,14 @@ def normalize_sender(sender):
       outside the directory.
     - Spaces are replaced with underscores.
     - The address is lowercased.
+    - Truncate sender at 233 chars to insure the full filename
+    (including time, pid, and two dots) fits within the POSIX limit of
+    255 chars for a filename.
     """
     sender = sender.replace(' ', '_')
     sender = sender.replace('/', ':')
-    return sender.lower()
+    sender = sender.lower()
+    return sender[:POSIX_NAME_MAX - 22]
 
 
 def confirm_append_address(xp, rp):
@@ -425,10 +466,12 @@ def confirm_append_address(xp, rp):
     rp is the envelope sender address.
 
     Compare the two addresses, and return the address appropriate for
-    CONFIRM_APPEND use based on the PRIMARY_ADDRESS_MATCH setting.
+    CONFIRM_APPEND based on the PRIMARY_ADDRESS_MATCH setting.
     """
     if not xp:
         return rp
+    if '@' not in rp or '@' not in xp:
+        return rp
     import Defaults
     rpl = rp.lower()
     xpl = xp.lower()
@@ -468,6 +511,21 @@ def confirm_append_address(xp, rp):
     return rp
 
 
+def msg_from_file(fp, strict=0):
+    """Read a file and parse its contents into a Message object model.
+    Replacement for email.message_from_file().
+    
+    We use the HeaderParser subclass instead of Parser to avoid trying
+    to parse the message body, instead setting the payload to the raw
+    body as a string.  This is faster, and also helps us avoid
+    problems trying to parse spam with broken MIME bodies."""
+    from email.Message import Message
+    from email.Parser import HeaderParser
+    msg = HeaderParser(Message, strict=strict).parse(fp)
+    msg.header_parsed = 1
+    return msg
+
+
 def msg_as_string(msg, maxheaderlen=0, mangle_from_=0, unixfrom=0):
     """A more flexible replacement for Message.as_string().  The default
     is a textual representation of the message where the headers are
@@ -485,8 +543,13 @@ def msg_as_string(msg, maxheaderlen=0, mangle_from_=0, unixfrom=0):
 
     unixfrom forces the printing of the envelope header delimiter.
     Default is False."""
+    from email import Generator
     fp = StringIO()
-    g = Generator(fp, mangle_from_=mangle_from_, maxheaderlen=maxheaderlen)
+    if hasattr(msg, 'header_parsed') and msg.header_parsed:
+        genclass = Generator.HeaderParsedGenerator
+    else:
+        genclass = Generator.Generator
+    g = genclass(fp, mangle_from_=mangle_from_, maxheaderlen=maxheaderlen)
     g.flatten(msg, unixfrom=unixfrom)
     return fp.getvalue()
 
@@ -502,36 +565,43 @@ def sendmail(msgstr, envrecip, envsender):
     envsender is the envelope sender address.
     """
     import Defaults
-    if Defaults.OUTGOINGMAIL == 'smtp':
+    if Defaults.MAIL_TRANSPORT == 'sendmail':
+        # You can avoid the shell by passing a tuple of arguments as
+        # the command instead of a string.  This will cause the
+        # popen2.Popen3() code to execvp() "/usr/bin/sendmail" with
+        # these arguments exactly, with no trip through any shell.
+        cmd = (Defaults.SENDMAIL_PROGRAM, '-i',
+               '-f', envsender, '--', envrecip)
+        pipecmd(cmd, msgstr)
+    elif Defaults.MAIL_TRANSPORT == 'smtp':
         import SMTP
         server = SMTP.Connection()
         server.sendmail(envsender, envrecip, msgstr)
         server.quit()
-    elif Defaults.OUTGOINGMAIL == 'sendmail':
-        cmd = "%s -f '%s' -- '%s'" % (Defaults.SENDMAIL_PROGRAM,
-                                      envsender, envrecip)
-        pipecmd(cmd, msgstr)
     else:
         raise Errors.ConfigError, \
-              "Invalid OUTGOINGMAIL method: " + Defaults.OUTGOINGMAIL
+              "Invalid MAIL_TRANSPORT method: " + Defaults.MAIL_TRANSPORT
 
 
 def decode_header(str):
     """Accept a possibly encoded message header as a string, and
-    return a decoded string.
+    return a decoded string if it can be decoded.
 
     JRM: email.Header has a decode_header method, but it returns a
     list of decoded pairs, one for each part of the header, which is
     an awkward interface IMO, especially when the header contains a
     mix of encoded and non-encoded parts.
     """
-    from email import Header
-    parts = []
-    pairs = Header.decode_header(str)
-    for pair in pairs:
-        parts.append(pair[0])
-    decoded_string = ' '.join(parts)
-    return decoded_string
+    try:
+        from email import Header
+        parts = []
+        pairs = Header.decode_header(str)
+        for pair in pairs:
+            parts.append(pair[0])
+        decoded_string = ' '.join(parts)
+        return decoded_string
+    except email.Errors.HeaderParseError:
+        return str
 
 
 def headers_as_list(msg):
@@ -583,6 +653,33 @@ def rename_headers(msg, old, new):
                 msg._headers[index] = (new, '%s' % pair[1])
 
 
+def add_headers(msg, headers):
+    """Add headers to a Message object.
+
+       msg is an email.Message.Message object.
+
+       headers is a dictionary of headers and values.
+       """
+    if headers:
+        keys = headers.keys()
+        keys.sort()
+        for k in keys:
+            del msg[k]
+            msg[k] = headers[k]
+
+
+def purge_headers(msg, headers):
+    """Purge headers from a Message object.
+
+       msg is an email.Message.Message object.
+
+       headers is a list of headers.
+       """
+    if headers:
+        for h in headers:
+            del msg[h]
+
+
 def build_cdb(filename):
     """Build a cdb file from a text file."""
     import cdb
@@ -656,6 +753,21 @@ def unpickle(file):
     return object
 
 
+def db_insert(db, insert_sql, params):
+    """Insert (using the 'insert_sql' SQL) an address into a SQL DB."""
+    dbmodule = sys.modules[db.__module__]
+    DatabaseError = getattr(dbmodule, 'DatabaseError')
+    cursor = db.cursor()
+    try:
+        try:
+            cursor.execute(insert_sql, params)
+            db.commit()
+        except DatabaseError:
+            pass
+    finally:
+        cursor.close()
+
+
 def findmatch(list, addrs):
     """Determine whether any of the passed e-mail addresses match a
     Unix shell-style wildcard pattern contained in list.  The
@@ -663,13 +775,13 @@ def findmatch(list, addrs):
     the string if it exists (for exp and ext addresses only)."""
     for address in addrs:
         if address:
-            address = string.lower(address)
+            address = address.lower()
             for p in list:
-                stringparts = string.split(p)
+                stringparts = p.split()
                 p = stringparts[0]
                 # Handle special @=domain.dom syntax.
                 try:
-                    at = string.rindex(p, '@')
+                    at = p.rindex('@')
                     atequals = p[at+1] == '='
                 except (ValueError, IndexError):
                     atequals = None
@@ -769,12 +881,19 @@ def wraptext(text, column=70, honor_leading_ws=1):
 def maketext(templatefile, vardict):
     """Make some text from a template file.
 
-     Several locations are scanned for templatefile, in the following order:
-     1. The directory specified by tmda-filter's `-t' option.
-     2. Defaults.TEMPLATE_DIR_MATCH_SENDER (if true)
-     3. Defaults.TEMPLATE_DIR
-     4. ../templates/
-     5. The package/RPM template directories.
+    templatefile can either be an absolute pathname starting with an /
+    or ~ (e.g, /usr/local/packages/tmda/templates/bounce.txt) or a
+    relative pathname (e.g, bounce.txt).
+
+    Given a relative pathname, several locations are scanned for
+    templatefile, in the following order:
+
+    1. The directory specified by tmda-filter's `-t' option.
+    2. Defaults.TEMPLATE_DIR_MATCH_SENDER (if true)
+    3. Defaults.TEMPLATE_DIR_MATCH_RECIPIENT (if true)
+    4. Defaults.TEMPLATE_DIR
+    5. ../templates/
+    6. The package/RPM template directories.
 
     The first match found stops the search.  In this way, you can
     specialize templates at the desired level, or, if you use only the
@@ -789,29 +908,57 @@ def maketext(templatefile, vardict):
     and licensed under the GNU General Public License version 2.
     """
     import Defaults
-    # Calculate the locations to scan.
-    searchdirs = []
-    searchdirs.append(os.environ.get('TMDA_TEMPLATE_DIR'))
-    if Defaults.TEMPLATE_DIR_MATCH_SENDER and Defaults.TEMPLATE_DIR:
-        sender = os.environ.get('SENDER').lower()
-        searchdirs.append(os.path.join(Defaults.TEMPLATE_DIR, sender))
-        domainparts = sender.split('@', 1)[1].split('.')
-        for i in range(len(domainparts)):
-            searchdirs.append(os.path.join
-                              (Defaults.TEMPLATE_DIR, '.'.join(domainparts)))
-            del domainparts[0]
-    searchdirs.append(Defaults.TEMPLATE_DIR)
-    searchdirs.append(os.path.join(Defaults.PARENTDIR, 'templates'))
-    searchdirs.append(os.path.join(sys.prefix, 'share/tmda'))
-    searchdirs.append('/etc/tmda/')
-    # Start scanning.
     foundit = None
-    for dir in searchdirs:
-        if dir:
-            filename = os.path.join(dir, templatefile)
-            if os.path.exists(filename):
-                foundit = filename
-                break
+    if (templatefile[0] == '/' or templatefile[0] == '~'):
+        if templatefile[0] == '~':
+            templatefile = os.path.expanduser(templatefile)
+        if os.path.exists(templatefile):
+            foundit = templatefile
+    else:
+        # Calculate the locations to scan.
+        searchdirs = []
+        searchdirs.append(os.environ.get('TMDA_TEMPLATE_DIR'))
+        if Defaults.TEMPLATE_DIR_MATCH_SENDER and Defaults.TEMPLATE_DIR:
+            sender = os.environ.get('SENDER').lower()
+            searchdirs.append(os.path.join(Defaults.TEMPLATE_DIR, sender))
+            try:
+                domainparts = sender.split('@', 1)[1].split('.')
+                for i in range(len(domainparts)):
+                    searchdirs.append(os.path.join
+                                      (Defaults.TEMPLATE_DIR, '.'.join(domainparts)))
+                    del domainparts[0]
+            except IndexError:
+                pass
+        if Defaults.TEMPLATE_DIR_MATCH_RECIPIENT and Defaults.TEMPLATE_DIR:
+            recipient = os.environ.get('TMDA_RECIPIENT').lower()
+            searchdirs.append(os.path.join(Defaults.TEMPLATE_DIR, recipient))
+            try:
+                recippart, domainpart = recipient.split('@',1)
+                recipparts = recippart.split(Defaults.RECIPIENT_DELIMITER)
+                for i in range(len(recipparts)):
+                    searchdirs.append(os.path.join
+                                      (Defaults.TEMPLATE_DIR,
+                                       Defaults.RECIPIENT_DELIMITER.join(recipparts) +
+                                       "@" + domainpart))
+                    del recipparts[-1]
+                domainparts = domainpart.split('.')
+                for i in range(len(domainparts)):
+                    searchdirs.append(os.path.join
+                                      (Defaults.TEMPLATE_DIR, '.'.join(domainparts)))
+                    del domainparts[0]
+            except IndexError:
+                pass
+        searchdirs.append(Defaults.TEMPLATE_DIR)
+        searchdirs.append(os.path.join(Defaults.PARENTDIR, 'templates'))
+        searchdirs.append(os.path.join(sys.prefix, 'share/tmda'))
+        searchdirs.append('/etc/tmda/')
+        # Start scanning.
+        for dir in searchdirs:
+            if dir:
+                filename = os.path.join(dir, templatefile)
+                if os.path.exists(filename):
+                    foundit = filename
+                    break
     if foundit is None:
         raise IOError, "Can't find " + templatefile
     else:
@@ -826,8 +973,9 @@ def maketext(templatefile, vardict):
 
 def filter_match(filename, recip, sender=None):
     """Check if the give e-mail addresses match lines in filename."""
+    import Defaults
     import FilterParser
-    filter = FilterParser.FilterParser()
+    filter = FilterParser.FilterParser(Defaults.DB_CONNECTION)
     filter.read(filename)
     (actions, matchline) = filter.firstmatch(recip, [sender])
     # print the results
@@ -884,9 +1032,9 @@ def CanMode( file, mode = MODE_READ, uid = None, gid = None ):
         uid = os.geteuid()
     if gid is None:
         gid = os.getegid()
-    needuid = fstat.st_uid
-    needgid = fstat.st_gid
-    filemod = fstat.st_mode & 0777
+    needuid = fstat[4]
+    needgid = fstat[5]
+    filemod = fstat[0] & 0777
     if uid == 0:
         # Root always wins.
         return 1
