@@ -1,6 +1,6 @@
 # -*- python -*-
 #
-# Copyright (C) 2001,2002 Jason R. Mastaler <jason@mastaler.com>
+# Copyright (C) 2001,2002,2003 Jason R. Mastaler <jason@mastaler.com>
 #
 # This file is part of TMDA.
 #
@@ -26,21 +26,25 @@ import sys
 
 
 # TMDA version
-TMDA = "0.68"
+TMDA = "1.0.2"
 
 # TMDA version codename
-CODENAME = "Shut Out"
+CODENAME = "Bold Forbes"
 
 # Python version
 PYTHON = sys.version.split()[0]
 
 # Platform identifier
 try:
-    from distutils.util import get_platform
-    PLATFORM = get_platform()
+    from platform import platform
+    PLATFORM = platform()
 except ImportError:
-    PLATFORM = sys.platform
+    try:
+        from distutils.util import get_platform
+        PLATFORM = get_platform()
+    except ImportError:
+        PLATFORM = sys.platform
 
 # Summary of all the version identifiers
-# e.g, TMDA/0.52 (Python 2.2.1 on osf1-V5.1-alpha)
-ALL = "TMDA/%s (Python %s on %s)" % (TMDA, PYTHON, PLATFORM)
+# e.g, TMDA/0.86+ "Carry Back" (Python/2.3.2 on Darwin-6.8-Power_Macintosh-powerpc-32bit)
+ALL = 'TMDA/%s "%s" (Python/%s on %s)' % (TMDA, CODENAME, PYTHON, PLATFORM)
